@@ -53,86 +53,17 @@
 #include <QStyleOption>
 #include <QPushButton>
 #include <QToolBar>
+#include <QShortcut>
 
 #include <kconfiggroup.h>
 #include <kiconloader.h>
+#include <KMessageWidget>
 #include <kcolorscheme.h>
-#include <config-kstyle.h>
-#if HAVE_X11
-#include <QX11Info>
-#include <xcb/xcb.h>
-#endif
 
 // ----------------------------------------------------------------------------
 
 static const QStyle::StyleHint SH_KCustomStyleElement = (QStyle::StyleHint)0xff000001;
 static const int X_KdeBase = 0xff000000;
-static const char s_schemePropertyName[] = "KDE_COLOR_SCHEME_PATH";
-
-/**
- * An event filter intended to be used for all toplevel QWidgets.
- * The filter checks for Show and PaletteChange events and passes the
- * KDE_COLOR_SCHEME_PATH property on the QApplication to the native window.
- * The application property is set by e.g. the KColorSchemeManager and the
- * information is needed by KDE's window manager to use the same color scheme
- * on e.g. the window decoration to provide a consistent look and feel.
- */
-class ColorSchemeFilter : public QObject
-{
-public:
-    ColorSchemeFilter(QObject *parent = Q_NULLPTR);
-    bool eventFilter(QObject *object, QEvent *event) Q_DECL_OVERRIDE;
-private:
-    void installColorScheme(QWidget *w);
-};
-
-ColorSchemeFilter::ColorSchemeFilter(QObject *parent)
-    : QObject(parent)
-{
-}
-
-bool ColorSchemeFilter::eventFilter(QObject *object, QEvent *event)
-{
-    QWidget *w = static_cast<QWidget*>(object);
-    if ((event->type() == QEvent::Show && qApp->property(s_schemePropertyName).isValid()) ||
-            (event->type() == QEvent::PaletteChange)) {
-        installColorScheme(w);
-    }
-    return QObject::eventFilter(object, event);
-}
-
-void ColorSchemeFilter::installColorScheme(QWidget *w)
-{
-    if (!w || !w->isTopLevel()) {
-        return;
-    }
-#if HAVE_X11
-    static const bool s_x11 = QX11Info::isPlatformX11();
-    if (!s_x11) {
-        return;
-    }
-    static xcb_atom_t atom = XCB_ATOM_NONE;
-    xcb_connection_t *c = QX11Info::connection();
-    if (atom == XCB_ATOM_NONE) {
-        const QByteArray name = QByteArrayLiteral("_KDE_NET_WM_COLOR_SCHEME");
-        const xcb_intern_atom_cookie_t cookie = xcb_intern_atom(c, false, name.length(), name.constData());
-        QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> reply(xcb_intern_atom_reply(c, cookie, Q_NULLPTR));
-        if (!reply.isNull()) {
-            atom = reply->atom;
-        } else {
-            // no point in continuing, we don't have the atom
-            return;
-        }
-    }
-    const QString path = qApp->property(s_schemePropertyName).toString();
-    if (path.isEmpty()) {
-        xcb_delete_property(c, w->winId(), atom);
-    } else {
-        xcb_change_property(c, XCB_PROP_MODE_REPLACE, w->winId(), atom, XCB_ATOM_STRING,
-                            8, path.size(), qPrintable(path));
-    }
-#endif
-}
 
 class KStylePrivate
 {
@@ -141,11 +72,9 @@ public:
 
     QHash<QString, int> styleElements;
     int hintCounter, controlCounter, subElementCounter;
-    QScopedPointer<ColorSchemeFilter> colorSchemeFilter;
 };
 
 KStylePrivate::KStylePrivate()
-    : colorSchemeFilter(new ColorSchemeFilter())
 {
     controlCounter = subElementCounter = X_KdeBase;
     hintCounter = X_KdeBase + 1; //sic! X_KdeBase is covered by SH_KCustomStyleElement
@@ -164,7 +93,7 @@ static inline int customStyleElement(QStyle::StyleHint type, const QString &elem
 
     const QString originalName = widget->objectName();
     widget->setObjectName(element);
-    const int id = widget->style()->styleHint(type, 0, widget);
+    const int id = widget->style()->styleHint(type, nullptr, widget);
     widget->setObjectName(originalName);
     return id;
 }
@@ -274,16 +203,33 @@ void KStyle::polish(QWidget *w)
 
     if (QDialogButtonBox *box = qobject_cast<QDialogButtonBox *>(w)) {
         QPushButton *button = box->button(QDialogButtonBox::Ok);
-        if (button && button->shortcut().isEmpty()) {
-            button->setShortcut(Qt::CTRL | Qt::Key_Return);
+
+        if (button) {
+            auto shortcut = new QShortcut(Qt::CTRL | Qt::Key_Return, button);
+            QObject::connect(shortcut, &QShortcut::activated, button, &QPushButton::click);
         }
     }
-
-    // install the event filter to pass the color scheme app property to all toplevel native windows
-    if (w->isTopLevel()) {
-        w->installEventFilter(d->colorSchemeFilter.data());
+    if (auto messageWidget = qobject_cast<KMessageWidget*>(w)) {
+        KColorScheme scheme;
+        QColor color;
+        QPalette palette = messageWidget->palette();
+        switch (messageWidget->messageType()) {
+        case KMessageWidget::Positive:
+            color = scheme.foreground(KColorScheme::PositiveText).color();
+            break;
+        case KMessageWidget::Information:
+            color = scheme.foreground(KColorScheme::ActiveText).color();
+            break;
+        case KMessageWidget::Warning:
+            color = scheme.foreground(KColorScheme::NeutralText).color();
+            break;
+        case KMessageWidget::Error:
+            color = scheme.foreground(KColorScheme::NegativeText).color();
+            break;
+        }
+        palette.setColor(QPalette::Window, color);
+        messageWidget->setPalette(palette);
     }
-
     QCommonStyle::polish(w);
 }
 
@@ -350,7 +296,8 @@ QIcon KStyle::standardIcon(StandardPixmap standardIcon, const QStyleOption *opti
     case QStyle::SP_MessageBoxCritical:
         return QIcon::fromTheme(QStringLiteral("dialog-error"));
     case QStyle::SP_MessageBoxQuestion:
-        return QIcon::fromTheme(QStringLiteral("dialog-information"));
+        // This used to be dialog-information for a long time, so keep it as a fallback
+        return QIcon::fromTheme(QStringLiteral("dialog-question"), QIcon::fromTheme(QStringLiteral("dialog-information")));
     case QStyle::SP_DialogOkButton:
         return QIcon::fromTheme(QStringLiteral("dialog-ok"));
     case QStyle::SP_DialogCancelButton:
@@ -366,9 +313,9 @@ QIcon KStyle::standardIcon(StandardPixmap standardIcon, const QStyleOption *opti
     case QStyle::SP_DialogApplyButton:
         return QIcon::fromTheme(QStringLiteral("dialog-ok-apply"));
     case QStyle::SP_DialogResetButton:
-        return QIcon::fromTheme(QStringLiteral("document-revert"));
+        return QIcon::fromTheme(QStringLiteral("edit-undo"));
     case QStyle::SP_DialogDiscardButton:
-        return QIcon::fromTheme(QStringLiteral("dialog-cancel"));
+        return QIcon::fromTheme(QStringLiteral("edit-delete"));
     case QStyle::SP_DialogYesButton:
         return QIcon::fromTheme(QStringLiteral("dialog-ok-apply"));
     case QStyle::SP_DialogNoButton:
@@ -407,6 +354,30 @@ QIcon KStyle::standardIcon(StandardPixmap standardIcon, const QStyleOption *opti
         return QIcon::fromTheme(QStringLiteral("audio-volume-medium"));
     case QStyle::SP_MediaVolumeMuted:
         return QIcon::fromTheme(QStringLiteral("audio-volume-muted"));
+    case SP_LineEditClearButton: {
+        const bool rtl = (option && option->direction == Qt::RightToLeft) || (!option && QApplication::isRightToLeft());
+
+        const QString directionalThemeName = rtl
+            ? QStringLiteral("edit-clear-locationbar-ltr") : QStringLiteral("edit-clear-locationbar-rtl");
+
+        return QIcon::fromTheme(directionalThemeName, QIcon::fromTheme(QStringLiteral("edit-clear")));
+    }
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    case QStyle::SP_DialogYesToAllButton:
+        return QIcon::fromTheme(QStringLiteral("dialog-ok"));
+    case QStyle::SP_DialogNoToAllButton:
+        return QIcon::fromTheme(QStringLiteral("dialog-cancel"));
+    case QStyle::SP_DialogSaveAllButton:
+        return QIcon::fromTheme(QStringLiteral("document-save-all"));
+    case QStyle::SP_DialogAbortButton:
+        return QIcon::fromTheme(QStringLiteral("dialog-cancel"));
+    case QStyle::SP_DialogRetryButton:
+        return QIcon::fromTheme(QStringLiteral("view-refresh"));
+    case QStyle::SP_DialogIgnoreButton:
+        return QIcon::fromTheme(QStringLiteral("dialog-cancel"));
+    case QStyle::SP_RestoreDefaultsButton:
+        return QIcon::fromTheme(QStringLiteral("document-revert"));
+#endif
 
     default:
         break;
@@ -432,9 +403,8 @@ int KStyle::styleHint(StyleHint hint, const QStyleOption *option, const QWidget
         return g.readEntry("GraphicEffectsLevel", true);
     }
 
-    #if QT_VERSION >= QT_VERSION_CHECK(5,5,0)
-    case QStyle::SH_Menu_SubMenuSloppyCloseTimeout: return 300;
-    #endif
+    case QStyle::SH_Menu_SubMenuSloppyCloseTimeout:
+        return 300;
 
     case SH_ToolButtonStyle: {
         KConfigGroup g(KSharedConfig::openConfig(), "Toolbar style");
@@ -471,6 +441,11 @@ int KStyle::styleHint(StyleHint hint, const QStyleOption *option, const QWidget
 
         return d->styleElements.value(widget->objectName(), 0);
 
+    case SH_ScrollBar_LeftClickAbsolutePosition: {
+        KConfigGroup g(KSharedConfig::openConfig(), "KDE");
+        return !g.readEntry("ScrollbarLeftClickNavigatesByPage", true);
+    }
+
     default:
         break;
     };
